@@ -1,6 +1,7 @@
 package cmd
 
 import (
+	"encoding/json"
 	"fmt"
 	"io"
 	"os"
@@ -19,74 +20,54 @@ import (
 )
 
 func (cli *CLI) inspect(opts Options) int {
-	// Respect the "--format" flag until a config is loaded
-	cli.formatter.Format = opts.Format
-
-	workingDirs, err := findWorkingDirs(opts)
-	if err != nil {
-		cli.formatter.Print(tflint.Issues{}, fmt.Errorf("Failed to find workspaces; %w", err), map[string][]byte{})
-		return ExitCodeError
-	}
-
 	issues := tflint.Issues{}
 	changes := map[string][]byte{}
 
-	for _, wd := range workingDirs {
-		err := cli.withinChangedDir(wd, func() error {
-			filterFiles := []string{}
-			for _, pattern := range opts.Filter {
-				files, err := filepath.Glob(pattern)
-				if err != nil {
-					return fmt.Errorf("Failed to parse --filter options; %w", err)
-				}
-				// Add the raw pattern to return an empty result if it doesn't match any files
-				if len(files) == 0 {
-					filterFiles = append(filterFiles, pattern)
-				}
-				filterFiles = append(filterFiles, files...)
-			}
-
-			// Join with the working directory to create the fullpath
-			for i, file := range filterFiles {
-				filterFiles[i] = filepath.Join(wd, file)
-			}
-
-			moduleIssues, moduleChanges, err := cli.inspectModule(opts, ".", filterFiles)
+	err := cli.withinChangedDir(opts.Chdir, func() error {
+		filterFiles := []string{}
+		for _, pattern := range opts.Filter {
+			files, err := filepath.Glob(pattern)
 			if err != nil {
-				return err
+				return fmt.Errorf("Failed to parse --filter options; %w", err)
 			}
-			issues = append(issues, moduleIssues...)
-			for path, source := range moduleChanges {
-				changes[path] = source
+			// Add the raw pattern to return an empty result if it doesn't match any files
+			if len(files) == 0 {
+				filterFiles = append(filterFiles, pattern)
 			}
+			filterFiles = append(filterFiles, files...)
+		}
 
-			return nil
-		})
+		// Join with the working directory to create the fullpath
+		for i, file := range filterFiles {
+			filterFiles[i] = filepath.Join(opts.Chdir, file)
+		}
+
+		var err error
+		issues, changes, err = cli.inspectModule(opts, ".", filterFiles)
+		return err
+	})
+	if err != nil {
+		sources := map[string][]byte{}
+		if cli.loader != nil {
+			sources = cli.loader.Sources()
+		}
+		cli.formatter.Print(tflint.Issues{}, err, sources)
+		return ExitCodeError
+	}
+
+	if opts.ActAsWorker {
+		// When acting as a recursive inspection worker, the formatter is ignored
+		// and the serialized issues are output.
+		out, err := json.Marshal(issues)
 		if err != nil {
-			sources := map[string][]byte{}
-			if cli.loader != nil {
-				sources = cli.loader.Sources()
-			}
-			cli.formatter.Print(tflint.Issues{}, err, sources)
+			fmt.Fprint(cli.errStream, err)
 			return ExitCodeError
 		}
-	}
-
-	var force bool
-	if opts.Recursive {
-		// Respect "--format" and "--force" flags in recursive mode
-		cli.formatter.Format = opts.Format
-		if opts.Force != nil {
-			force = *opts.Force
-		}
+		fmt.Fprint(cli.outStream, string(out))
 	} else {
-		cli.formatter.Format = cli.config.Format
-		force = cli.config.Force
+		cli.formatter.Print(issues, nil, cli.sources)
 	}
 
-	cli.formatter.Fix = opts.Fix
-	cli.formatter.Print(issues, nil, cli.sources)
-
 	if opts.Fix {
 		if err := writeChanges(changes); err != nil {
 			cli.formatter.Print(tflint.Issues{}, err, cli.sources)
@@ -94,7 +75,7 @@ func (cli *CLI) inspect(opts Options) int {
 		}
 	}
 
-	if len(issues) > 0 && !force && exceedsMinimumFailure(issues, opts.MinimumFailureSeverity) {
+	if len(issues) > 0 && !cli.config.Force && exceedsMinimumFailure(issues, opts.MinimumFailureSeverity) {
 		return ExitCodeIssuesFound
 	}
 
@@ -118,22 +99,25 @@ func (cli *CLI) inspectModule(opts Options, dir string, filterFiles []string) (t
 	if err != nil {
 		return issues, changes, fmt.Errorf("Failed to prepare loading; %w", err)
 	}
-	if opts.Recursive && !cli.loader.IsConfigDir(dir) {
-		// Ignore non-module directories in recursive mode
+	if opts.ActAsWorker && !cli.loader.IsConfigDir(dir) {
+		// Ignore non-module directories in worker mode
 		return issues, changes, nil
 	}
 
 	// Setup runners
-	runners, err := cli.setupRunners(opts, dir)
+	rootRunner, moduleRunners, err := cli.setupRunners(opts, dir)
 	if err != nil {
 		return issues, changes, err
 	}
-	rootRunner := runners[len(runners)-1]
 
 	// Launch plugin processes
 	rulesetPlugin, err := launchPlugins(cli.config, opts.Fix)
 	if rulesetPlugin != nil {
 		defer rulesetPlugin.Clean()
+		go cli.registerShutdownHandler(func() {
+			rulesetPlugin.Clean()
+			os.Exit(ExitCodeError)
+		})
 	}
 	if err != nil {
 		return issues, changes, err
@@ -172,16 +156,33 @@ By setting TFLINT_LOG=trace, you can confirm the changes made by the autofix and
 		}
 
 		for name, ruleset := range rulesetPlugin.RuleSets {
-			for _, runner := range runners {
-				err = ruleset.Check(plugin.NewGRPCServer(runner, rootRunner, cli.loader.Files(), sdkVersions[name]))
+			if err := ruleset.Check(plugin.NewGRPCServer(rootRunner, rootRunner, cli.loader.Files(), sdkVersions[name])); err != nil {
+				return issues, changes, fmt.Errorf("Failed to check ruleset; %w", err)
+			}
+			// Run checks for module calls are performed in parallel.
+			// The rootRunner is shared between goroutines but read-only, so this is goroutine-safe.
+			// Note that checks against the rootRunner are not parallelized, as autofix may cause the module to be rebuilt.
+			ch := make(chan error, len(moduleRunners))
+			for _, runner := range moduleRunners {
+				if opts.NoParallelRunners {
+					ch <- ruleset.Check(plugin.NewGRPCServer(runner, rootRunner, cli.loader.Files(), sdkVersions[name]))
+				} else {
+					go func(runner *tflint.Runner) {
+						ch <- ruleset.Check(plugin.NewGRPCServer(runner, rootRunner, cli.loader.Files(), sdkVersions[name]))
+					}(runner)
+				}
+			}
+			for i := 0; i < len(moduleRunners); i++ {
+				err = <-ch
 				if err != nil {
 					return issues, changes, fmt.Errorf("Failed to check ruleset; %w", err)
 				}
 			}
+			close(ch)
 		}
 
 		changesInAttempt := map[string][]byte{}
-		for _, runner := range runners {
+		for _, runner := range append(moduleRunners, rootRunner) {
 			for _, issue := range runner.LookupIssues(filterFiles...) {
 				// On the second attempt, only fixable issues are appended to avoid duplicates.
 				if loop == 1 || issue.Fixable {
@@ -210,15 +211,15 @@ By setting TFLINT_LOG=trace, you can confirm the changes made by the autofix and
 	return issues, changes, nil
 }
 
-func (cli *CLI) setupRunners(opts Options, dir string) ([]*tflint.Runner, error) {
-	configs, diags := cli.loader.LoadConfig(dir, cli.config.Module)
+func (cli *CLI) setupRunners(opts Options, dir string) (*tflint.Runner, []*tflint.Runner, error) {
+	configs, diags := cli.loader.LoadConfig(dir, cli.config.CallModuleType)
 	if diags.HasErrors() {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to load configurations; %w", diags)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to load configurations; %w", diags)
 	}
 
 	files, diags := cli.loader.LoadConfigDirFiles(dir)
 	if diags.HasErrors() {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to load configurations; %w", diags)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to load configurations; %w", diags)
 	}
 	annotations := map[string]tflint.Annotations{}
 	for path, file := range files {
@@ -230,30 +231,30 @@ func (cli *CLI) setupRunners(opts Options, dir string) ([]*tflint.Runner, error)
 		annotations[path] = ants
 	}
 	if diags.HasErrors() {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to load configurations; %w", diags)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to load configurations; %w", diags)
 	}
 
 	variables, diags := cli.loader.LoadValuesFiles(dir, cli.config.Varfiles...)
 	if diags.HasErrors() {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to load values files; %w", diags)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to load values files; %w", diags)
 	}
 	cliVars, diags := terraform.ParseVariableValues(cli.config.Variables, configs.Module.Variables)
 	if diags.HasErrors() {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to parse variables; %w", diags)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to parse variables; %w", diags)
 	}
 	variables = append(variables, cliVars)
 
 	runner, err := tflint.NewRunner(cli.originalWorkingDir, cli.config, annotations, configs, variables...)
 	if err != nil {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to initialize a runner; %w", err)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to initialize a runner; %w", err)
 	}
 
-	runners, err := tflint.NewModuleRunners(runner)
+	moduleRunners, err := tflint.NewModuleRunners(runner)
 	if err != nil {
-		return []*tflint.Runner{}, fmt.Errorf("Failed to prepare rule checking; %w", err)
+		return nil, []*tflint.Runner{}, fmt.Errorf("Failed to prepare rule checking; %w", err)
 	}
 
-	return append(runners, runner), nil
+	return runner, moduleRunners, nil
 }
 
 func launchPlugins(config *tflint.Config, fix bool) (*plugin.Plugin, error) {
@@ -275,7 +276,7 @@ func launchPlugins(config *tflint.Config, fix bool) (*plugin.Plugin, error) {
 				// VersionConstraints endpoint is available in tflint-plugin-sdk v0.14+.
 				return rulesetPlugin, fmt.Errorf(`Plugin "%s" SDK version is incompatible. Compatible versions: %s`, name, plugin.SDKVersionConstraints)
 			} else {
-				return rulesetPlugin, fmt.Errorf("Failed to get TFLint version constraints to `%s` plugin; %w", name, err)
+				return rulesetPlugin, fmt.Errorf(`Failed to get TFLint version constraints to "%s" plugin; %w`, name, err)
 			}
 		}
 		if !constraints.Check(tflint.Version) {
@@ -283,23 +284,23 @@ func launchPlugins(config *tflint.Config, fix bool) (*plugin.Plugin, error) {
 		}
 
 		if err := ruleset.ApplyGlobalConfig(pluginConf); err != nil {
-			return rulesetPlugin, fmt.Errorf("Failed to apply global config to `%s` plugin; %w", name, err)
+			return rulesetPlugin, fmt.Errorf(`Failed to apply global config to "%s" plugin; %w`, name, err)
 		}
 		configSchema, err := ruleset.ConfigSchema()
 		if err != nil {
-			return rulesetPlugin, fmt.Errorf("Failed to fetch config schema from `%s` plugin; %w", name, err)
+			return rulesetPlugin, fmt.Errorf(`Failed to fetch config schema from "%s" plugin; %w`, name, err)
 		}
 		content := &hclext.BodyContent{}
 		if plugin, exists := config.Plugins[name]; exists {
 			var diags hcl.Diagnostics
 			content, diags = plugin.Content(configSchema)
 			if diags.HasErrors() {
-				return rulesetPlugin, fmt.Errorf("Failed to parse `%s` plugin config; %w", name, diags)
+				return rulesetPlugin, fmt.Errorf(`Failed to parse "%s" plugin config; %w`, name, diags)
 			}
 		}
 		err = ruleset.ApplyConfig(content, config.Sources())
 		if err != nil {
-			return rulesetPlugin, fmt.Errorf("Failed to apply config to `%s` plugin; %w", name, err)
+			return rulesetPlugin, fmt.Errorf(`Failed to apply config to "%s" plugin; %w`, name, err)
 		}
 
 		rulesets = append(rulesets, ruleset)
