@@ -2,6 +2,7 @@ package plugin
 
 import (
 	plugin "github.com/hashicorp/go-plugin"
+	"github.com/hashicorp/go-version"
 	"github.com/terraform-linters/tflint-plugin-sdk/plugin/host2plugin"
 )
 
@@ -12,10 +13,13 @@ var (
 	localPluginRoot = "./.tflint.d/plugins"
 )
 
+// SDKVersionConstraints is the version constraint of the supported SDK version.
+var SDKVersionConstraints = version.MustConstraints(version.NewConstraint(">= 0.14.0"))
+
 // Plugin is an object handling plugins
 // Basically, it is a wrapper for go-plugin and provides an API to handle them collectively.
 type Plugin struct {
-	RuleSets map[string]*host2plugin.GRPCClient
+	RuleSets map[string]*host2plugin.Client
 
 	clients map[string]*plugin.Client
 }
