@@ -142,6 +142,27 @@ variable "string_var" {
 			want:     `cty.StringVal("acbd18db4cc2f85cedef654fccc4a4d8")`,
 			errCheck: neverHappend,
 		},
+		{
+			name:     "built-in function with namespace",
+			expr:     expr(`core::md5("foo")`),
+			ty:       cty.String,
+			want:     `cty.StringVal("acbd18db4cc2f85cedef654fccc4a4d8")`,
+			errCheck: neverHappend,
+		},
+		{
+			name:     "provider-defined functions",
+			expr:     expr(`provider::tflint::echo("Hello", "World!")`),
+			ty:       cty.String,
+			want:     `cty.UnknownVal(cty.String)`,
+			errCheck: neverHappend,
+		},
+		{
+			name:     "built-in provider-defined functions",
+			expr:     expr(`provider::terraform::tfvarsdecode("a = 1")`),
+			ty:       cty.Object(map[string]cty.Type{"a": cty.Number}),
+			want:     `cty.ObjectVal(map[string]cty.Value{"a":cty.NumberIntVal(1)})`,
+			errCheck: neverHappend,
+		},
 		{
 			name:     "terraform workspace",
 			expr:     expr(`terraform.workspace`),
@@ -322,7 +343,7 @@ variable "null_var" {
 			config:   `variable "no_value_var" {}`,
 			expr:     expr(`"Hello, ${var.no_value_var}"`),
 			ty:       cty.String,
-			want:     `cty.UnknownVal(cty.String)`,
+			want:     `cty.UnknownVal(cty.String).Refine().NotNull().StringPrefixFull("Hello, ").NewValue()`,
 			errCheck: neverHappend,
 		},
 		{
@@ -348,7 +369,7 @@ variable "null_var" {
 			name:     "interpolation with unevalauble value",
 			expr:     expr(`"Hello, ${module.text}"`),
 			ty:       cty.String,
-			want:     `cty.UnknownVal(cty.String)`,
+			want:     `cty.UnknownVal(cty.String).Refine().NotNull().StringPrefixFull("Hello, ").NewValue()`,
 			errCheck: neverHappend,
 		},
 		{
@@ -928,6 +949,19 @@ resource "aws_instance" "main" {
 			config: `
 resource "aws_instance" "main" {
   count = module.meta.count
+}`,
+			schema: &hclext.BodySchema{
+				Blocks: []hclext.BlockSchema{
+					{Type: "resource", LabelNames: []string{"type", "name"}, Body: &hclext.BodySchema{}},
+				},
+			},
+			want: &hclext.BodyContent{Attributes: hclext.Attributes{}, Blocks: hclext.Blocks{}},
+		},
+		{
+			name: "count is using provider-defined functions",
+			config: `
+resource "aws_instance" "main" {
+  count = provider::tflint::count()
 }`,
 			schema: &hclext.BodySchema{
 				Blocks: []hclext.BlockSchema{
@@ -989,7 +1023,7 @@ resource "aws_instance" "main" {
 					{
 						Type:   "resource",
 						Labels: []string{"aws_instance", "main"},
-						Body:   &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).Mark(marks.Sensitive), hcl.Range{})}}, Blocks: hclext.Blocks{}},
+						Body:   &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).RefineNotNull().Mark(marks.Sensitive), hcl.Range{})}}, Blocks: hclext.Blocks{}},
 					},
 				},
 			},
@@ -1021,6 +1055,29 @@ resource "aws_instance" "main" {
 				},
 			},
 		},
+		{
+			name: "count.index and provider-defined functions",
+			config: `
+resource "aws_instance" "main" {
+  count = 1
+  value = [count.index, provider::tflint::sum(1, 2, 3)]
+}`,
+			schema: &hclext.BodySchema{
+				Blocks: []hclext.BlockSchema{
+					{Type: "resource", LabelNames: []string{"type", "name"}, Body: &hclext.BodySchema{Attributes: []hclext.AttributeSchema{{Name: "value"}}}},
+				},
+			},
+			want: &hclext.BodyContent{
+				Attributes: hclext.Attributes{},
+				Blocks: hclext.Blocks{
+					{
+						Type:   "resource",
+						Labels: []string{"aws_instance", "main"},
+						Body:   &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.TupleVal([]cty.Value{cty.NumberIntVal(0), cty.DynamicVal}), hcl.Range{})}}, Blocks: hclext.Blocks{}},
+					},
+				},
+			},
+		},
 		{
 			name: "for_each is not empty (literal)",
 			config: `
@@ -1086,10 +1143,23 @@ resource "aws_instance" "main" {
 			want: &hclext.BodyContent{Attributes: hclext.Attributes{}, Blocks: hclext.Blocks{}},
 		},
 		{
-			name: "for_each is evaluable",
+			name: "for_each is unevaluable",
 			config: `
 resource "aws_instance" "main" {
   for_each = module.meta.for_each
+}`,
+			schema: &hclext.BodySchema{
+				Blocks: []hclext.BlockSchema{
+					{Type: "resource", LabelNames: []string{"type", "name"}, Body: &hclext.BodySchema{}},
+				},
+			},
+			want: &hclext.BodyContent{Attributes: hclext.Attributes{}, Blocks: hclext.Blocks{}},
+		},
+		{
+			name: "for_each is using provider-defined functions",
+			config: `
+resource "aws_instance" "main" {
+  for_each = provider::tflint::for_each()
 }`,
 			schema: &hclext.BodySchema{
 				Blocks: []hclext.BlockSchema{
@@ -1221,7 +1291,7 @@ resource "aws_instance" "main" {
 					{
 						Type:   "resource",
 						Labels: []string{"aws_instance", "main"},
-						Body:   &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).Mark(marks.Sensitive), hcl.Range{})}}, Blocks: hclext.Blocks{}},
+						Body:   &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).RefineNotNull().Mark(marks.Sensitive), hcl.Range{})}}, Blocks: hclext.Blocks{}},
 					},
 				},
 			},
@@ -1415,6 +1485,47 @@ resource "aws_instance" "main" {
       value = "${ebs_block_device.key}-${ebs_block_device.value}"
     }
   }
+}`,
+			schema: &hclext.BodySchema{
+				Blocks: []hclext.BlockSchema{
+					{
+						Type:       "resource",
+						LabelNames: []string{"type", "name"},
+						Body: &hclext.BodySchema{
+							Blocks: []hclext.BlockSchema{
+								{
+									Type: "ebs_block_device",
+									Body: &hclext.BodySchema{Attributes: []hclext.AttributeSchema{{Name: "value"}}},
+								},
+							},
+						},
+					},
+				},
+			},
+			want: &hclext.BodyContent{
+				Attributes: hclext.Attributes{},
+				Blocks: hclext.Blocks{
+					{
+						Type:   "resource",
+						Labels: []string{"aws_instance", "main"},
+						Body: &hclext.BodyContent{
+							Attributes: hclext.Attributes{},
+							Blocks:     hclext.Blocks{},
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "dynamic blocks with provider-defined functions",
+			config: `
+resource "aws_instance" "main" {
+  dynamic "ebs_block_device" {
+    for_each = provider::tflint::for_each()
+    content {
+      value = "${ebs_block_device.key}-${ebs_block_device.value}"
+    }
+  }
 }`,
 			schema: &hclext.BodySchema{
 				Blocks: []hclext.BlockSchema{
@@ -1491,7 +1602,7 @@ resource "aws_instance" "main" {
 								},
 								{
 									Type: "ebs_block_device",
-									Body: &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String), hcl.Range{})}}, Blocks: hclext.Blocks{}},
+									Body: &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).Refine().NotNull().StringPrefixFull("unknown-").NewValue(), hcl.Range{})}}, Blocks: hclext.Blocks{}},
 								},
 							},
 						},
@@ -1673,7 +1784,7 @@ resource "aws_instance" "main" {
 							Blocks: hclext.Blocks{
 								{
 									Type: "ebs_block_device",
-									Body: &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).Mark(marks.Sensitive), hcl.Range{})}}, Blocks: hclext.Blocks{}},
+									Body: &hclext.BodyContent{Attributes: hclext.Attributes{"value": {Name: "value", Expr: hcl.StaticExpr(cty.UnknownVal(cty.String).RefineNotNull().Mark(marks.Sensitive), hcl.Range{})}}, Blocks: hclext.Blocks{}},
 								},
 							},
 						},
